@@ -589,6 +589,30 @@ class ALEX_bursts(ALEX_data):
         self.burst_ends = np.array(burst_ends)
         self.burst_len = self.burst_ends - self.burst_starts
 
+    def denoise_bursts(self, N_DD, N_DA, N_AD, N_AA):
+        """
+        Subtract background noise from ALEX bursts.
+
+        Arguments:
+
+        * N_DD: average noise per time-bin in the channel D_D
+        * N_DA: average noise per time-bin in the channel D_A
+        * N_AD: average noise per time-bin in the channel A_D
+        * N_AA: average noise per time-bin in the channel A_A
+        """
+        DD_noise = N_DD * self.burst_len
+        DA_noise = N_DA * self.burst_len
+        AD_noise = N_AD * self.burst_len
+        AA_noise = N_AA * self.burst_len
+        acceptor_noise = N_A * self.burst_len
+
+        self.D_D = self.D_D - DD_noise
+        self.D_A = self.D_A - DA_noise
+        self.A_D = self.A_D - AD_noise
+        self.A_A = self.A_A - AA_noise
+
+        return self
+
     def scatter_intensity(self, filepath, imgname, imgtype="pdf", labels = ["Burst Duration", "Burst Intensity"]):
         """
         Plot a scatter plot of burst brightness vs burst duration
