@@ -491,6 +491,24 @@ class FRET_bursts(FRET_data):
         self.burst_ends = burst_ends
         self.burst_len = self.burst_ends - self.burst_starts
 
+    def denoise_bursts(self, N_D, N_A):
+        """
+        Subtract background noise from donor and acceptor bursts.
+
+        Arguments:
+
+        * N_D: average noise per time-bin in the donor channel
+        * N_A: average noise per time-bin in the acceptor channel
+        """
+        
+        donor_noise = N_D * self.burst_len
+        acceptor_noise = N_A * self.burst_len
+
+        self.donor = self.donor - donor_noise
+        self.acceptor = self.acceptor - acceptor_noise
+
+        return self
+
     def scatter_intensity(self, filepath, imgname, imgtype="pdf", labels = ["Burst Duration", "Burst Intensity"]):
         """
         Plot a scatter plot of burst brightness vs burst duration
@@ -513,6 +531,54 @@ class FRET_bursts(FRET_data):
         plt.close()
         return self
 
+    def RASP(self, Emin, Emax, Tmin, Tmax, gamma=1.0):
+        """
+        Recurrence Analysis of Single Particles analysis as implemented in Hoffmann et al. Phys Chem Chem Phys. 2011 13(5):1857-1871.
+        Returns a FRET_bursts object.
+
+        Arguments:
+        * Emin: minimum value of E (proximity ratio) to consider for initial bursts
+        * Emax: maximum value of E (proximity ratio) to consider for initial bursts
+        * Tmin: start time (in number of bins after the initial burst) to search for recurrent bursts 
+        * Tmax: end time (in number of bins after the initial burst) to search for recurrent bursts
+        
+        Keyword Arguments:
+        * gamma: value of instrumental gamma factor to use in calculating the proximity ratio. Default value = 1.0.
+
+        From Hoffmann et al.:
+        First, the bursts b2 must be detected during a time interval between t1 and t2 (the 'recurrence interval', T = (t1,t2)) 
+        after a previous burst b1 (the 'initial burst'). Second, the initial bursts must yield a transfer efficiency, E(b1), 
+        within a defined range, Delta E1 (the 'initial E range'). 
+
+        In this implementation, Tmin and Tmax correspond to t1 and t2 respectively. The initial E range lies between Emin and Emax.
+
+        """
+
+        E1 = self.proximity_ratio(gamma)
+        E_select = (E1 > Emin) & (E1 < Emax)
+
+        E_sel = E1[E_select]
+        end_sel = self.burst_ends[E_select]
+
+        T_start = end_sel + Tmin
+        T_end = end_sel + Tmax
+
+        Eevents = np.array([])
+        donor = np.array([])
+        acceptor = np.array([])
+        b_start = np.array([])
+        b_end = np.array([])
+
+        for s, e in zip(T_start, T_end):
+            sel = (self.burst_starts >= s) & (self.burst_starts <= e)
+            Eevents = np.hstack((Eevents, E1[sel]))
+            donor = np.hstack((donor, self.donor[sel]))
+            acceptor = np.hstack((acceptor, self.acceptor[sel]))
+            b_start = np.hstack((b_start, self.burst_starts[sel]))
+            b_end = np.hstack((b_end, self.burst_ends[sel]))
+        recurrent_bursts = FRET_bursts(donor, acceptor, b_start, b_end)
+        return recurrent_bursts
+
 
 def parse_csv(filepath, filelist, delimiter=","):
     """
